@@ -205,6 +205,9 @@ class SVG:
             self.elements[idx] = (el, (shape.as_path(),))
         return self
 
+    def _apply_styles(self, el: etree.Element):
+        parse_css_declarations(el.attrib.pop("style", ""), el.attrib)
+
     def apply_style_attributes(self, inplace=False):
         """Converts inlined CSS "style" attributes to equivalent SVG attributes."""
         if not inplace:
@@ -220,7 +223,7 @@ class SVG:
 
         # parse all remaining style attributes (e.g. in gradients or root svg element)
         for el in itertools.chain((self.svg_root,), self.xpath("//svg:*[@style]")):
-            parse_css_declarations(el.attrib.pop("style", ""), el.attrib)
+            self._apply_styles(el)
 
         return self
 
@@ -356,6 +359,7 @@ class SVG:
             "clip-path": _inherit_clip_path,
             "id": lambda *_: 0,
             "data-name": lambda *_: 0,
+            "enable-background": lambda *_: 0,
             "overflow": _inherit_nondefault_overflow,
         }
 
@@ -386,6 +390,9 @@ class SVG:
         groups = [e for e in self.xpath(f".//svg:g", scope_el)]
         multi_clips = []
         for group in groups:
+            # sneaky little styles, we hates it precious
+            self._apply_styles(group)
+
             # move groups children up
             # reverse because "for each addnext" effectively reverses
             children = list(group)
@@ -415,7 +422,9 @@ class SVG:
             clip_paths = [self._resolve_clip_path(ref) for ref in clip_ref_urls]
             clip_path = self._combine_clip_paths(clip_paths)
             if clip_path.d not in new_clip_paths:
-                new_el = etree.SubElement(self.svg_root, "clipPath")
+                new_el = etree.SubElement(
+                    self.svg_root, f"{{{svgns()}}}clipPath", nsmap=self.svg_root.nsmap
+                )
                 new_el.attrib["id"] = self._new_id("clipPath", "merged-clip-%d")
                 new_el.append(to_element(clip_path))
                 new_clip_paths[clip_path.d] = new_el
@@ -599,6 +608,16 @@ class SVG:
                 shape.remove_overlaps(inplace=True)
         return self
 
+    def round_floats(self, ndigits: int, inplace=False):
+        if not inplace:
+            svg = SVG(copy.deepcopy(self.svg_root))
+            svg.round_floats(ndigits, inplace=True)
+            return svg
+
+        for shape in self.shapes():
+            shape.round_floats(ndigits, inplace=True)
+        return self
+
     def remove_unpainted_shapes(self, inplace=False):
         if not inplace:
             svg = SVG(copy.deepcopy(self.svg_root))
@@ -701,7 +720,7 @@ class SVG:
 
         return tuple(errors)
 
-    def topicosvg(self, inplace=False):
+    def topicosvg(self, *, ndigits=3, inplace=False):
         if not inplace:
             svg = SVG(copy.deepcopy(self.svg_root))
             svg.topicosvg(inplace=True)
@@ -714,12 +733,14 @@ class SVG:
         self.shapes_to_paths(inplace=True)
         self.resolve_use(inplace=True)
         self.apply_transforms(inplace=True)
-        self.apply_clip_paths(inplace=True)
         self.ungroup(inplace=True)
         # stroke after ungroup to apply group strokes properly
         self.strokes_to_paths(inplace=True)
+        self.apply_clip_paths(inplace=True)
         self.evenodd_to_nonzero_winding(inplace=True)
         self.remove_unpainted_shapes(inplace=True)
+        self.absolute(inplace=True)
+        self.round_floats(ndigits, inplace=True)
 
         # Collect gradients; remove other defs
         defs = etree.Element(f"{{{svgns()}}}defs", nsmap=self.svg_root.nsmap)
