@@ -23,6 +23,75 @@ from picosvg.svg_transform import Affine2D
 from typing import Generator, Iterable
 
 
+def _explicit_lines_callback(subpath_start, curr_pos, cmd, args, *_):
+    del subpath_start
+    if cmd == "v":
+        args = (0, args[0])
+    elif cmd == "V":
+        args = (curr_pos.x, args[0])
+    elif cmd == "h":
+        args = (args[0], 0)
+    elif cmd == "H":
+        args = (args[0], curr_pos.y)
+    else:
+        return ((cmd, args),)  # nothing changes
+
+    if cmd.islower():
+        cmd = "l"
+    else:
+        cmd = "L"
+
+    return ((cmd, args),)
+
+
+def _relative_to_absolute(curr_pos, cmd, args):
+    x_coord_idxs, y_coord_idxs = svg_meta.cmd_coords(cmd)
+    if cmd.islower():
+        cmd = cmd.upper()
+        args = list(args)  # we'd like to mutate 'em
+        for x_coord_idx in x_coord_idxs:
+            args[x_coord_idx] += curr_pos.x
+        for y_coord_idx in y_coord_idxs:
+            args[y_coord_idx] += curr_pos.y
+
+    return (cmd, tuple(args))
+
+
+def _next_pos(curr_pos, cmd, cmd_args):
+    # update current position
+    x_coord_idxs, y_coord_idxs = svg_meta.cmd_coords(cmd)
+    new_x, new_y = curr_pos
+    if cmd.isupper():
+        if x_coord_idxs:
+            new_x = 0
+        if y_coord_idxs:
+            new_y = 0
+
+    if x_coord_idxs:
+        new_x += cmd_args[x_coord_idxs[-1]]
+    if y_coord_idxs:
+        new_y += cmd_args[y_coord_idxs[-1]]
+
+    return Point(new_x, new_y)
+
+
+def _move_endpoint(curr_pos, cmd, cmd_args, new_endpoint):
+    # we need to be able to alter both axes
+    ((cmd, cmd_args),) = _explicit_lines_callback(None, curr_pos, cmd, cmd_args)
+
+    x_coord_idxs, y_coord_idxs = svg_meta.cmd_coords(cmd)
+    cmd_args = list(cmd_args)  # we'd like to mutate
+    new_x, new_y = new_endpoint
+    if cmd.islower():
+        new_x = new_x - curr_pos.x
+        new_y = new_y - curr_pos.y
+
+    cmd_args[x_coord_idxs[-1]] = new_x
+    cmd_args[y_coord_idxs[-1]] = new_y
+
+    return cmd, cmd_args
+
+
 # Subset of https://www.w3.org/TR/SVG11/painting.html
 @dataclasses.dataclass
 class SVGShape:
@@ -158,6 +227,21 @@ class SVGShape:
             target.style = unparsed_style
         return target
 
+    def round_floats(self, ndigits: int, inplace=False) -> "SVGShape":
+        """Round all floats in SVGShape to given decimal digits."""
+        target = self
+        if not inplace:
+            target = copy.deepcopy(self)
+        for field in dataclasses.fields(target):
+            field_value = getattr(self, field.name)
+            if isinstance(field_value, float):
+                setattr(target, field.name, round(field_value, ndigits))
+        return target
+
+    def almost_equals(self, other: "SVGShape", tolerance: int) -> bool:
+        assert isinstance(other, SVGShape)
+        return self.round_floats(tolerance) == other.round_floats(tolerance)
+
 
 # https://www.w3.org/TR/SVG11/paths.html#PathElement
 @dataclasses.dataclass
@@ -238,11 +322,12 @@ class SVGPath(SVGShape, svg_meta.SVGCommandSeq):
 
         https://www.w3.org/TR/SVG11/paths.html
 
-        def callback(curr_xy, cmd, args, prev_xy, prev_cmd, prev_args)
+        def callback(subpath_start, curr_xy, cmd, args, prev_xy, prev_cmd, prev_args)
           prev_* None if there was no previous
           returns sequence of (new_cmd, new_args) that replace cmd, args
         """
         curr_pos = Point()
+        subpath_start_pos = curr_pos  # where a z will take you
         new_cmds = []
 
         # iteration gives us exploded commands
@@ -254,35 +339,29 @@ class SVGPath(SVGShape, svg_meta.SVGCommandSeq):
             prev = (None, None, None)
             if new_cmds:
                 prev = new_cmds[-1]
-            for (new_cmd, new_cmd_args) in callback(curr_pos, cmd, args, *prev):
-                # update current position
-                x_coord_idxs, y_coord_idxs = svg_meta.cmd_coords(new_cmd)
-                new_x = curr_pos.x
-                new_y = curr_pos.y
-                if new_cmd.isupper():
-                    if x_coord_idxs:
-                        new_x = 0
-                    if y_coord_idxs:
-                        new_y = 0
+            for (new_cmd, new_cmd_args) in callback(
+                subpath_start_pos, curr_pos, cmd, args, *prev
+            ):
+                if new_cmd.lower() != "z":
+                    next_pos = _next_pos(curr_pos, new_cmd, new_cmd_args)
+                else:
+                    next_pos = subpath_start_pos
 
-                if x_coord_idxs:
-                    new_x += new_cmd_args[x_coord_idxs[-1]]
-                if y_coord_idxs:
-                    new_y += new_cmd_args[y_coord_idxs[-1]]
-
-                prev_pos = copy.copy(curr_pos)
-                curr_pos = Point(new_x, new_y)
+                prev_pos, curr_pos = curr_pos, next_pos
+                if new_cmd.upper() == "M":
+                    subpath_start_pos = curr_pos
                 new_cmds.append((prev_pos, new_cmd, new_cmd_args))
 
         self.d = ""
         for _, cmd, args in new_cmds:
             self._add_cmd(cmd, *args)
 
-    # TODO replace with a proper transform
     def move(self, dx, dy, inplace=False):
         """Returns a new path that is this one shifted."""
 
-        def move_callback(_, cmd, args, *_unused):
+        def move_callback(subpath_start, curr_pos, cmd, args, *_unused):
+            del subpath_start
+            del curr_pos
             # Paths must start with an absolute moveto. Relative bits are ... relative.
             # Shift the absolute parts and call it a day.
             if cmd.islower():
@@ -301,23 +380,20 @@ class SVGPath(SVGShape, svg_meta.SVGCommandSeq):
         target.walk(move_callback)
         return target
 
-    @staticmethod
-    def _relative_to_absolute(curr_pos, cmd, args):
-        x_coord_idxs, y_coord_idxs = svg_meta.cmd_coords(cmd)
-        if cmd.islower():
-            cmd = cmd.upper()
-            args = list(args)  # we'd like to mutate 'em
-            for x_coord_idx in x_coord_idxs:
-                args[x_coord_idx] += curr_pos.x
-            for y_coord_idx in y_coord_idxs:
-                args[y_coord_idx] += curr_pos.y
-        return (cmd, tuple(args))
-
     def absolute(self, inplace=False) -> "SVGPath":
         """Returns equivalent path with only absolute commands."""
 
-        def absolute_callback(curr_pos, cmd, args, *_):
-            return (SVGPath._relative_to_absolute(curr_pos, cmd, args),)
+        def absolute_callback(subpath_start, curr_pos, cmd, args, *_):
+            new_cmd, new_cmd_args = _relative_to_absolute(curr_pos, cmd, args)
+
+            # if we modified cmd to pass *very* close to subpath start snap to it
+            # eliminates issues with not-quite-closed shapes due float imprecision
+            next_pos = _next_pos(curr_pos, new_cmd, new_cmd_args)
+            if next_pos != subpath_start and next_pos.almost_equals(subpath_start):
+                new_cmd, new_cmd_args = _move_endpoint(
+                    curr_pos, new_cmd, new_cmd_args, subpath_start
+                )
+            return ((new_cmd, new_cmd_args),)
 
         target = self
         if not inplace:
@@ -327,30 +403,10 @@ class SVGPath(SVGShape, svg_meta.SVGCommandSeq):
 
     def explicit_lines(self, inplace=False):
         """Replace all vertical/horizontal lines with line to (x,y)."""
-
-        def explicit_line_callback(curr_pos, cmd, args, *_):
-            if cmd == "v":
-                args = (0, args[0])
-            elif cmd == "V":
-                args = (curr_pos.x, args[0])
-            elif cmd == "h":
-                args = (args[0], 0)
-            elif cmd == "H":
-                args = (args[0], curr_pos.y)
-            else:
-                return ((cmd, args),)  # nothing changes
-
-            if cmd.islower():
-                cmd = "l"
-            else:
-                cmd = "L"
-
-            return ((cmd, args),)
-
         target = self
         if not inplace:
             target = copy.deepcopy(self)
-        target.walk(explicit_line_callback)
+        target.walk(_explicit_lines_callback)
         return target
 
     def expand_shorthand(self, inplace=False):
@@ -362,20 +418,20 @@ class SVGPath(SVGShape, svg_meta.SVGCommandSeq):
         """
 
         def expand_shorthand_callback(
-            curr_pos, cmd, args, prev_pos, prev_cmd, prev_args
+            _, curr_pos, cmd, args, prev_pos, prev_cmd, prev_args
         ):
             short_to_long = {"S": "C", "T": "Q"}
             if not cmd.upper() in short_to_long:
                 return ((cmd, args),)
 
             if cmd.islower():
-                cmd, args = SVGPath._relative_to_absolute(curr_pos, cmd, args)
+                cmd, args = _relative_to_absolute(curr_pos, cmd, args)
 
             # if there is no prev, or a bad prev, control point coincident current
             new_cp = (curr_pos.x, curr_pos.y)
             if prev_cmd:
                 if prev_cmd.islower():
-                    prev_cmd, prev_args = SVGPath._relative_to_absolute(
+                    prev_cmd, prev_args = _relative_to_absolute(
                         prev_pos, prev_cmd, prev_args
                     )
                 if prev_cmd in short_to_long.values():
@@ -394,7 +450,8 @@ class SVGPath(SVGShape, svg_meta.SVGCommandSeq):
     def arcs_to_cubics(self, inplace=False):
         """Replace all arcs with similar cubics"""
 
-        def arc_to_cubic_callback(curr_pos, cmd, args, *_):
+        def arc_to_cubic_callback(subpath_start, curr_pos, cmd, args, *_):
+            del subpath_start
             if cmd not in {"a", "A"}:
                 # no work to do
                 return ((cmd, args),)
@@ -445,6 +502,19 @@ class SVGPath(SVGShape, svg_meta.SVGCommandSeq):
             target._add_cmd(cmd, *args)
         return target
 
+    def round_floats(self, ndigits: int, inplace=False) -> "SVGPath":
+        """Round all floats in SVGPath to given decimal digits.
+
+        Also reformat the SVGPath.d string floats with the same rounding.
+        """
+        target: SVGPath = super().round_floats(ndigits, inplace=inplace)
+
+        d, target.d = target.d, ""
+        for cmd, args in parse_svg_path(d):
+            target._add_cmd(cmd, *(round(n, ndigits) for n in args))
+
+        return target
+
 
 # https://www.w3.org/TR/SVG11/shapes.html#CircleElement
 @dataclasses.dataclass
